# -*- coding: utf-8 -*-
nds20 = \
    """0 0 0 0 0 
0 0 0 0 0 
0 0 0 0 0 
0 0 0 0 0 


0 0 0 0 0 
0 0 0 1 0 
5 4 3 2 1 
0 0 0 3 0 


0 0 0 0 0 
0 0 0 0 0 
0 0 0 0 0 
0 0 0 0 0 


#=== dimension 4

0 0 0 0 0 
0 0 0 0 0 
0 0 0 0 0 
0 0 0 0 0 


0 0 0 0 0 
0 0 0 0 0 
0 0 0 0 0 
0 0 0 0 0 


0 0 0 0 0 
0 0 0 0 0 
0 0 0 0 0 
0 0 0 0 0 


#=== dimension 4

"""

nds30 = \
    """0 0 0 0 
0 0 0 0 
0 0 0 0 
0 0 0 0 
0 0 0 0 


0 0 5 0 
0 0 4 0 
0 0 3 0 
0 1 2 3 
0 0 1 0 


0 0 0 0 
0 0 0 0 
0 0 0 0 
0 0 0 0 
0 0 0 0 


#=== dimension 4

0 0 0 0 
0 0 0 0 
0 0 0 0 
0 0 0 0 
0 0 0 0 


0 0 0 0 
0 0 0 0 
0 0 0 0 
0 0 0 0 
0 0 0 0 


0 0 0 0 
0 0 0 0 
0 0 0 0 
0 0 0 0 
0 0 0 0 


#=== dimension 4

"""

nds2 =\
    """0  0  0  0  0
0  0  0  0  0
0  0  0  0  0
0  0  0  0  0


0  0  0  0  0
0  0  0  1  0
5  4  3  2  1
0  0  0  3  0


0  0  0  0  0
0  0  0  0  0
0  0  0  0  0
0  0  0  0  0


#=== dimension 4

0  0  0  0  0
0  0  0  0  0
0  0  0  0  0
0  0  0  0  0


0  0  0  0  0
0  0  0  0  0
0  0  0  0  0
0  0  0  0  0


0  0  0  0  0
0  0  0  0  0
0  0  0  0  0
0  0  0  0  0


#=== dimension 4

"""

nds3 =\
    """0  0  0  0
0  0  0  0
0  0  0  0
0  0  0  0
0  0  0  0


0  0  5  0
0  0  4  0
0  0  3  0
0  1  2  3
0  0  1  0


0  0  0  0
0  0  0  0
0  0  0  0
0  0  0  0
0  0  0  0


#=== dimension 4

0  0  0  0
0  0  0  0
0  0  0  0
0  0  0  0
0  0  0  0


0  0  0  0
0  0  0  0
0  0  0  0
0  0  0  0
0  0  0  0


0  0  0  0
0  0  0  0
0  0  0  0
0  0  0  0
0  0  0  0


#=== dimension 4

"""
# -*- coding: utf-8 -*-
out_MetaData = """MetaData.toString with extra

+--------------+--------------------+--------+----------+---------+-----------------+--------+-------------------+-------------+--------------------+--------------------+-------------+
| name         | value              | unit   | type     | valid   | default         | code   | description       | fits_keyw   | id_zh_cn           | descripti          | valid_zh_   |
|              |                    |        |          |         |                 |        |                   | ord         |                    | on_zh_cn           | cn          |
+==============+====================+========+==========+=========+=================+========+===================+=============+====================+====================+=============+
| description  | MetaData.toString  |        | string   | None    | UNKNOWN         | B      | Description of th | DESC        | 描述               | 对本产品的描述。   |             |
|              | with extra         |        |          |         |                 |        | is product        |             |                    |                    |             |
+--------------+--------------------+--------+----------+---------+-----------------+--------+-------------------+-------------+--------------------+--------------------+-------------+
| type         | Product            |        | string   | None    | Product         | B      | Product Type iden | TYPE        | 产品类型           | 产品类型。完整Py   |             |
|              |                    |        |          |         |                 |        | tification. Name  |             |                    | thon类名或卡片   |             |
|              |                    |        |          |         |                 |        | of class or CARD. |             |                    | 名。          |             |
+--------------+--------------------+--------+----------+---------+-----------------+--------+-------------------+-------------+--------------------+--------------------+-------------+
| level        | ALL                |        | string   | None    | ALL             | B      | Product level.    | LEVEL       | 产品xx             | 产品xx             |             |
+--------------+--------------------+--------+----------+---------+-----------------+--------+-------------------+-------------+--------------------+--------------------+-------------+
| creator      | UNKNOWN            |        | string   | None    | UNKNOWN         | B      | Generator of this | CREATOR     | 本产品生成者       | 本产品生成方的标识 |             |
|              |                    |        |          |         |                 |        |  product.         |             |                    | ，例如可以是单位、 |             |
|              |                    |        |          |         |                 |        |                   |             |                    | 组织、姓名、软件、 |             |
|              |                    |        |          |         |                 |        |                   |             |                    | 或特别算法等。   |             |
+--------------+--------------------+--------+----------+---------+-----------------+--------+-------------------+-------------+--------------------+--------------------+-------------+
| creationDate | 1958-01-01T00:00:0 |        | finetime | None    | 1958-01-01T00:0 | Q      | Creation date of  | DATE        | 产品生成时间       | 本产品生成时间     |             |
|              | 0.000000           |        |          |         | 0:00.000000     |        | this product      |             |                    |                    |             |
|              | 0                  |        |          |         | 0               |        |                   |             |                    |                    |             |
+--------------+--------------------+--------+----------+---------+-----------------+--------+-------------------+-------------+--------------------+--------------------+-------------+
| rootCause    | UNKNOWN            |        | string   | None    | UNKNOWN         | B      | Reason of this ru | ROOTCAUS    | 数据来源           | 数据来源（此例来自 |             |
|              |                    |        |          |         |                 |        | n of pipeline.    |             |                    | 鉴定件热真空罐）  |             |
+--------------+--------------------+--------+----------+---------+-----------------+--------+-------------------+-------------+--------------------+--------------------+-------------+
| version      | 0.8                |        | string   | None    | 0.8             | B      | Version of produc | VERSION     | 版本               | 产品版本           |             |
|              |                    |        |          |         |                 |        | t                 |             |                    |                    |             |
+--------------+--------------------+--------+----------+---------+-----------------+--------+-------------------+-------------+--------------------+--------------------+-------------+
| FORMATV      | 1.6.0.10           |        | string   | None    | 1.6.0.10        | B      | Version of produc | FORMATV     | 格式版本           | 产品格式版本       |             |
|              |                    |        |          |         |                 |        | t schema and revi |             |                    |                    |             |
|              |                    |        |          |         |                 |        | sion              |             |                    |                    |             |
+--------------+--------------------+--------+----------+---------+-----------------+--------+-------------------+-------------+--------------------+--------------------+-------------+
| startDate    | 1958-01-01T00:00:0 |        | finetime | None    | 1958-01-01T00:0 | Q      | Nominal start tim | DATE-OBS    | 产品的标称起始时间 | 产品标称的起始时间 |             |
|              | 0.000000           |        |          |         | 0:00.000000     |        | e  of this produc |             |                    |                    |             |
|              | 0                  |        |          |         | 0               |        | t.                |             |                    |                    |             |
+--------------+--------------------+--------+----------+---------+-----------------+--------+-------------------+-------------+--------------------+--------------------+-------------+
| endDate      | 1958-01-01T00:00:0 |        | finetime | None    | 1958-01-01T00:0 | Q      | Nominal end time  | DATE-END    | 产品的标称结束时间 | 产品标称的结束时间 |             |
|              | 0.000000           |        |          |         | 0:00.000000     |        |  of this product. |             |                    |                    |             |
|              | 0                  |        |          |         | 0               |        |                   |             |                    |                    |             |
+--------------+--------------------+--------+----------+---------+-----------------+--------+-------------------+-------------+--------------------+--------------------+-------------+
| instrument   | UNKNOWN            |        | string   | None    | UNKNOWN         | B      | Instrument that g | INSTRUME    | 观测仪器名称       | 观测仪器名称       |             |
|              |                    |        |          |         |                 |        | enerated data of  |             |                    |                    |             |
|              |                    |        |          |         |                 |        | this product      |             |                    |                    |             |
+--------------+--------------------+--------+----------+---------+-----------------+--------+-------------------+-------------+--------------------+--------------------+-------------+
| modelName    | UNKNOWN            |        | string   | None    | UNKNOWN         | B      | Model name of the | MODEL       | 样机名称           | 观测仪器样机名称   |             |
|              |                    |        |          |         |                 |        |  instrument of th |             |                    |                    |             |
|              |                    |        |          |         |                 |        | is product        |             |                    |                    |             |
+--------------+--------------------+--------+----------+---------+-----------------+--------+-------------------+-------------+--------------------+--------------------+-------------+
| mission      | _AGS               |        | string   | None    | _AGS            | B      | Name of the missi | TELESCOP    | 任务名称           | 任务名称           |             |
|              |                    |        |          |         |                 |        | on.               |             |                    |                    |             |
+--------------+--------------------+--------+----------+---------+-----------------+--------+-------------------+-------------+--------------------+--------------------+-------------+
| listeners    | <No listener>      |        |          |         |                 |        |                   |             |                    |                    |             |
+--------------+--------------------+--------+----------+---------+-----------------+--------+-------------------+-------------+--------------------+--------------------+-------------+"""
out_Dataset = """level 0
                                              ******************************                                             
                                              *** Dataset (test Dataset) ***                                             
                                                           META                                                          
+-----------+--------------------+--------+----------+-------------------+-----------------+--------+-------------------+
| name      | value              | unit   | type     | valid             | default         | code   | description       |
+===========+====================+========+==========+===================+=================+========+===================+
| a         | 3.4                | None   | float    | (0, 31): valid    | 2.0             | None   | rule name, if is  |
|           |                    |        |          | 99:               |                 |        | "valid", "", or " |
|           |                    |        |          |                   |                 |        | default", is ommi |
|           |                    |        |          |                   |                 |        | ted in value stri |
|           |                    |        |          |                   |                 |        | ng.               |
+-----------+--------------------+--------+----------+-------------------+-----------------+--------+-------------------+
| b         | xy (2019-02-19T01: |        | finetime | (0, 9876543210123 | 1958-01-01T00:0 | Q      | date param        |
|           | 02:03.456789       |        |          | 456): xy          | 0:00.000099     |        |                   |
|           | 1929229360456789)  |        |          |                   | 99              |        |                   |
+-----------+--------------------+--------+----------+-------------------+-----------------+--------+-------------------+
| c         | Invalid (IJK)      |        | string   | '': empty         | cliche          | B      | this is a string  |
|           |                    |        |          |                   |                 |        | parameter. but on |
|           |                    |        |          |                   |                 |        | ly "" is allowed. |
+-----------+--------------------+--------+----------+-------------------+-----------------+--------+-------------------+
| d         | off (0b00)         | None   | binary   | 11000 0b01: on    | 0b0             | H      | valid rules descr |
|           |                    |        |          | 11000 0b00: off   |                 |        | ibed with binary  |
|           |                    |        |          |                   |                 |        | masks             |
+-----------+--------------------+--------+----------+-------------------+-----------------+--------+-------------------+
| listeners | <No listener>      |        |          |                   |                 |        |                   |
+-----------+--------------------+--------+----------+-------------------+-----------------+--------+-------------------+
                                                           DATA                                                          
                                                           ----                                                          
88.8
...

=========================================================================================================================

level 1, repr
               *** Dataset (test Dataset) ***              

-------------  --------------------------  ----------------
a= 3.4         b= xy (2019-02-19T01:02:03  c= Invalid (IJK)
               .456789
               1929229360456789)
d= off (0b00)  listeners= <No listener>
-------------  --------------------------  ----------------

                            DATA                           
                            ----                           
88.8
...

===========================================================

level 2,
Dataset{ 88.8, description = "test Dataset", meta = <MetaData a=(float: 3.4 <None>), b=(finetime: xy (FineTime(2019-02-19
01:02:03.456789)) <>), c=(string: Invalid (IJK) <>), d=(binary: off (0b00) <None>), ...> }"""
out_ArrayDataset = """

level 0
                                           *****************************************                                           
                                           *** ArrayDataset (toString tester AD) ***                                           
                                                              META                                                             
+-----------------+--------------------+--------+----------+-------------------+-----------------+--------+-------------------+
| name            | value              | unit   | type     | valid             | default         | code   | description       |
+=================+====================+========+==========+===================+=================+========+===================+
| description     | toString tester AD |        | string   | None              | UNKNOWN         | B      | Description of th |
|                 |                    |        |          |                   |                 |        | is dataset        |
+-----------------+--------------------+--------+----------+-------------------+-----------------+--------+-------------------+
| shape           | (2, 3, 4, 5)       |        | tuple    | None              | ()              |        | Number of element |
|                 |                    |        |          |                   |                 |        | s in each dimensi |
|                 |                    |        |          |                   |                 |        | on. Quick changer |
|                 |                    |        |          |                   |                 |        | s to the right.   |
+-----------------+--------------------+--------+----------+-------------------+-----------------+--------+-------------------+
| type            | ArrayDataset       |        | string   | None              | ArrayDataset    | B      | Data Type identif |
|                 |                    |        |          |                   |                 |        | ication.          |
+-----------------+--------------------+--------+----------+-------------------+-----------------+--------+-------------------+
| unit            | lyr                |        | string   | None              | None            | B      | Unit of every ele |
|                 |                    |        |          |                   |                 |        | ment.             |
+-----------------+--------------------+--------+----------+-------------------+-----------------+--------+-------------------+
| typecode        | UNKNOWN            |        | string   | None              | UNKNOWN         | B      | Python internal s |
|                 |                    |        |          |                   |                 |        | torage code.      |
+-----------------+--------------------+--------+----------+-------------------+-----------------+--------+-------------------+
| version         | 0.1                |        | string   | None              | 0.1             | B      | Version of datase |
|                 |                    |        |          |                   |                 |        | t                 |
+-----------------+--------------------+--------+----------+-------------------+-----------------+--------+-------------------+
| FORMATV         | 1.6.0.2            |        | string   | None              | 1.6.0.2         | B      | Version of datase |
|                 |                    |        |          |                   |                 |        | t schema and revi |
|                 |                    |        |          |                   |                 |        | sion              |
+-----------------+--------------------+--------+----------+-------------------+-----------------+--------+-------------------+
| a               | 3.4                | None   | float    | (0, 31): valid    | 2.0             | None   | rule name, if is  |
|                 |                    |        |          | 99:               |                 |        | "valid", "", or " |
|                 |                    |        |          |                   |                 |        | default", is ommi |
|                 |                    |        |          |                   |                 |        | ted in value stri |
|                 |                    |        |          |                   |                 |        | ng.               |
+-----------------+--------------------+--------+----------+-------------------+-----------------+--------+-------------------+
| b               | xy (2019-02-19T01: |        | finetime | (0, 9876543210123 | 1958-01-01T00:0 | Q      | date param        |
|                 | 02:03.456789       |        |          | 456): xy          | 0:00.000099     |        |                   |
|                 | 1929229360456789)  |        |          |                   | 99              |        |                   |
+-----------------+--------------------+--------+----------+-------------------+-----------------+--------+-------------------+
| c               | Invalid (IJK)      |        | string   | '': empty         | cliche          | B      | this is a string  |
|                 |                    |        |          |                   |                 |        | parameter. but on |
|                 |                    |        |          |                   |                 |        | ly "" is allowed. |
+-----------------+--------------------+--------+----------+-------------------+-----------------+--------+-------------------+
| d               | off (0b00)         | None   | binary   | 11000 0b01: on    | 0b0             | H      | valid rules descr |
|                 |                    |        |          | 11000 0b00: off   |                 |        | ibed with binary  |
|                 |                    |        |          |                   |                 |        | masks             |
+-----------------+--------------------+--------+----------+-------------------+-----------------+--------+-------------------+
| added_parameter | 42                 | None   | integer  | None              | None            | None   | A non-builtin par |
|                 |                    |        |          |                   |                 |        | am                |
+-----------------+--------------------+--------+----------+-------------------+-----------------+--------+-------------------+
| listeners       | <No listener>      |        |          |                   |                 |        |                   |
+-----------------+--------------------+--------+----------+-------------------+-----------------+--------+-------------------+
                                                              DATA                                                             
                                                              ----                                                             
0  0  0  0  0
0  0  0  0  0
0  0  0  0  0
0  0  0  0  0


0  0  0  0  0
0  0  0  1  0
5  4  3  2  1
0  0  0  3  0


0  0  0  0  0
0  0  0  0  0
0  0  0  0  0
0  0  0  0  0


#=== dimension 4

0  0  0  0  0
0  0  0  0  0
0  0  0  0  0
0  0  0  0  0


0  0  0  0  0
0  0  0  0  0
0  0  0  0  0
0  0  0  0  0


0  0  0  0  0
0  0  0  0  0
0  0  0  0  0
0  0  0  0  0


#=== dimension 4


===============================================================================================================================



level 1
                 *** ArrayDataset (toString tester AD) ***                 

--------------------------  -------------------  --------------------------
description= toString test  shape= (2, 3, 4, 5)  type= ArrayDataset
er AD
unit= lyr                   typecode= UNKNOWN    version= 0.1
FORMATV= 1.6.0.2            a= 3.4               b= xy (2019-02-19T01:02:03
                                                 .456789
                                                 1929229360456789)
c= Invalid (IJK)            d= off (0b00)        added_parameter= 42
listeners= <No listener>
--------------------------  -------------------  --------------------------

0  0  0  0  0
0  0  0  0  0
0  0  0  0  0
0  0  0  0  0


0  0  0  0  0
0  0  0  1  0
5  4  3  2  1
0  0  0  3  0


0  0  0  0  0
0  0  0  0  0
0  0  0  0  0
0  0  0  0  0


#=== dimension 4

0  0  0  0  0
0  0  0  0  0
0  0  0  0  0
0  0  0  0  0


0  0  0  0  0
0  0  0  0  0
0  0  0  0  0
0  0  0  0  0


0  0  0  0  0
0  0  0  0  0
0  0  0  0  0
0  0  0  0  0


#=== dimension 4


===========================================================================



level 2, repr
ArrayDataset(<MetaData description=(string: toString tester AD <>), shape=(tuple: (2, 3, 4, 5) <>), unit=(string: lyr <>), a=(float: 3.4 <None>), b=(finetime: xy (FineTime(2019-02-19
01:02:03.456789)) <>), c=(string: Invalid (IJK) <>), d=(binary: off (0b00) <None>), added_parameter=(integer: 42 <None>), ...> data= [[[[0, 0, 0, ...0]]], [[[0, 0, 0, ...0]]]])

an empty meta and long data level 2: 
ArrayDataset(<MetaData shape=(tuple: (8,) <>), ...> data= [8, 8, 8, 8, 8, 8, 8, 8])

"""
out_TableDataset = """

level 0
                                                 ******************************                                                
                                                 *** TableDataset (UNKNOWN) ***                                                
                                                              META                                                             
+-----------------+--------------------+--------+----------+-------------------+-----------------+--------+-------------------+
| name            | value              | unit   | type     | valid             | default         | code   | description       |
+=================+====================+========+==========+===================+=================+========+===================+
| description     | UNKNOWN            |        | string   | None              | UNKNOWN         | B      | Description of th |
|                 |                    |        |          |                   |                 |        | is dataset        |
+-----------------+--------------------+--------+----------+-------------------+-----------------+--------+-------------------+
| shape           | (2, 3)             |        | tuple    | None              | ()              |        | Number of columns |
|                 |                    |        |          |                   |                 |        |  and rows.        |
+-----------------+--------------------+--------+----------+-------------------+-----------------+--------+-------------------+
| type            | TableDataset       |        | string   | None              | TableDataset    | B      | Data Type identif |
|                 |                    |        |          |                   |                 |        | ication.          |
+-----------------+--------------------+--------+----------+-------------------+-----------------+--------+-------------------+
| version         | 0.1                |        | string   | None              | 0.1             | B      | Version of datase |
|                 |                    |        |          |                   |                 |        | t                 |
+-----------------+--------------------+--------+----------+-------------------+-----------------+--------+-------------------+
| FORMATV         | 1.6.0.2            |        | string   | None              | 1.6.0.2         | B      | Version of datase |
|                 |                    |        |          |                   |                 |        | t schema and revi |
|                 |                    |        |          |                   |                 |        | sion              |
+-----------------+--------------------+--------+----------+-------------------+-----------------+--------+-------------------+
| a               | 3.4                | None   | float    | (0, 31): valid    | 2.0             | None   | rule name, if is  |
|                 |                    |        |          | 99:               |                 |        | "valid", "", or " |
|                 |                    |        |          |                   |                 |        | default", is ommi |
|                 |                    |        |          |                   |                 |        | ted in value stri |
|                 |                    |        |          |                   |                 |        | ng.               |
+-----------------+--------------------+--------+----------+-------------------+-----------------+--------+-------------------+
| b               | xy (2019-02-19T01: |        | finetime | (0, 9876543210123 | 1958-01-01T00:0 | Q      | date param        |
|                 | 02:03.456789       |        |          | 456): xy          | 0:00.000099     |        |                   |
|                 | 1929229360456789)  |        |          |                   | 99              |        |                   |
+-----------------+--------------------+--------+----------+-------------------+-----------------+--------+-------------------+
| c               | Invalid (IJK)      |        | string   | '': empty         | cliche          | B      | this is a string  |
|                 |                    |        |          |                   |                 |        | parameter. but on |
|                 |                    |        |          |                   |                 |        | ly "" is allowed. |
+-----------------+--------------------+--------+----------+-------------------+-----------------+--------+-------------------+
| d               | off (0b00)         | None   | binary   | 11000 0b01: on    | 0b0             | H      | valid rules descr |
|                 |                    |        |          | 11000 0b00: off   |                 |        | ibed with binary  |
|                 |                    |        |          |                   |                 |        | masks             |
+-----------------+--------------------+--------+----------+-------------------+-----------------+--------+-------------------+
| added_parameter | 42                 | None   | integer  | None              | None            | None   | A non-builtin par |
|                 |                    |        |          |                   |                 |        | am                |
+-----------------+--------------------+--------+----------+-------------------+-----------------+--------+-------------------+
| listeners       | <No listener>      |        |          |                   |                 |        |                   |
+-----------------+--------------------+--------+----------+-------------------+-----------------+--------+-------------------+
                                                              DATA                                                             
                                                              ----                                                             
  col1     col2
  (eV)    (cnt)
------  -------
   1        0
   4.4     43.2
5400     2000

===============================================================================================================================



level 1
                     *** TableDataset (UNKNOWN) ***                     

--------------------------  ------------------------  ------------------
description= UNKNOWN        shape= (2, 3)             type= TableDataset
version= 0.1                FORMATV= 1.6.0.2          a= 3.4
b= xy (2019-02-19T01:02:03  c= Invalid (IJK)          d= off (0b00)
.456789
1929229360456789)
added_parameter= 42         listeners= <No listener>
--------------------------  ------------------------  ------------------

  col1     col2
  (eV)    (cnt)
------  -------
   1        0
   4.4     43.2
5400     2000

========================================================================



level 2, repr
TableDataset(<MetaData shape=(tuple: (2, 3) <>), a=(float: 3.4 <None>), b=(finetime: xy (FineTime(2019-02-19
01:02:03.456789)) <>), c=(string: Invalid (IJK) <>), d=(binary: off (0b00) <None>), added_parameter=(integer: 42 <None>), ...>data= {"col1": Column(<MetaData description=(string: 1 <>), shape=(tuple: (3,) <>), type=(string: Column <>), unit=(string: eV <>), ...> data= [1, 4.4, 5400.0]), "col2": Column(<MetaData description=(string: 2 <>), shape=(tuple: (3,) <>), type=(string: Column <>), unit=(string: cnt <>), ...> data= [0, 43.2, 2000.0])})

an empty level 2: 
TableDataset(<MetaData shape=(tuple: (0,) <>), ...>data= {})

"""
out_CompositeDataset = """level 0
                                            **********************************                                           
                                            *** CompositeDataset (test CD) ***                                           
                                                           META                                                          
+-----------+--------------------+--------+----------+-------------------+-----------------+--------+-------------------+
| name      | value              | unit   | type     | valid             | default         | code   | description       |
+===========+====================+========+==========+===================+=================+========+===================+
| a         | 3.4                | None   | float    | (0, 31): valid    | 2.0             | None   | rule name, if is  |
|           |                    |        |          | 99:               |                 |        | "valid", "", or " |
|           |                    |        |          |                   |                 |        | default", is ommi |
|           |                    |        |          |                   |                 |        | ted in value stri |
|           |                    |        |          |                   |                 |        | ng.               |
+-----------+--------------------+--------+----------+-------------------+-----------------+--------+-------------------+
| b         | xy (2019-02-19T01: |        | finetime | (0, 9876543210123 | 1958-01-01T00:0 | Q      | date param        |
|           | 02:03.456789       |        |          | 456): xy          | 0:00.000099     |        |                   |
|           | 1929229360456789)  |        |          |                   | 99              |        |                   |
+-----------+--------------------+--------+----------+-------------------+-----------------+--------+-------------------+
| c         | Invalid (IJK)      |        | string   | '': empty         | cliche          | B      | this is a string  |
|           |                    |        |          |                   |                 |        | parameter. but on |
|           |                    |        |          |                   |                 |        | ly "" is allowed. |
+-----------+--------------------+--------+----------+-------------------+-----------------+--------+-------------------+
| d         | off (0b00)         | None   | binary   | 11000 0b01: on    | 0b0             | H      | valid rules descr |
|           |                    |        |          | 11000 0b00: off   |                 |        | ibed with binary  |
|           |                    |        |          |                   |                 |        | masks             |
+-----------+--------------------+--------+----------+-------------------+-----------------+--------+-------------------+
| m1        | 2.3                | sec    | float    | None              | None            | None   | a different param |
|           |                    |        |          |                   |                 |        |  in metadata      |
+-----------+--------------------+--------+----------+-------------------+-----------------+--------+-------------------+
| listeners | <No listener>      |        |          |                   |                 |        |                   |
+-----------+--------------------+--------+----------+-------------------+-----------------+--------+-------------------+

                                      Total 2 Sub-Datasets: "dataset 1", "dataset 2"                                     

SubDataset  "dataset 1":
                                            __________________________________                                           
                                            *** ArrayDataset (arraydset 1) ***                                           
                                                           META                                                          
+-------------+---------------+--------+--------+---------+--------------+--------+-------------------+
| name        | value         | unit   | type   | valid   | default      | code   | description       |
+=============+===============+========+========+=========+==============+========+===================+
| description | arraydset 1   |        | string | None    | UNKNOWN      | B      | Description of th |
|             |               |        |        |         |              |        | is dataset        |
+-------------+---------------+--------+--------+---------+--------------+--------+-------------------+
| shape       | (3,)          |        | tuple  | None    | ()           |        | Number of element |
|             |               |        |        |         |              |        | s in each dimensi |
|             |               |        |        |         |              |        | on. Quick changer |
|             |               |        |        |         |              |        | s to the right.   |
+-------------+---------------+--------+--------+---------+--------------+--------+-------------------+
| type        | ArrayDataset  |        | string | None    | ArrayDataset | B      | Data Type identif |
|             |               |        |        |         |              |        | ication.          |
+-------------+---------------+--------+--------+---------+--------------+--------+-------------------+
| unit        | ev            |        | string | None    | None         | B      | Unit of every ele |
|             |               |        |        |         |              |        | ment.             |
+-------------+---------------+--------+--------+---------+--------------+--------+-------------------+
| typecode    | UNKNOWN       |        | string | None    | UNKNOWN      | B      | Python internal s |
|             |               |        |        |         |              |        | torage code.      |
+-------------+---------------+--------+--------+---------+--------------+--------+-------------------+
| version     | 0.1           |        | string | None    | 0.1          | B      | Version of datase |
|             |               |        |        |         |              |        | t                 |
+-------------+---------------+--------+--------+---------+--------------+--------+-------------------+
| FORMATV     | 1.6.0.2       |        | string | None    | 1.6.0.2      | B      | Version of datase |
|             |               |        |        |         |              |        | t schema and revi |
|             |               |        |        |         |              |        | sion              |
+-------------+---------------+--------+--------+---------+--------------+--------+-------------------+
| listeners   | <No listener> |        |        |         |              |        |                   |
+-------------+---------------+--------+--------+---------+--------------+--------+-------------------+
                                                           DATA                                                          
                                                           ----                                                          
768  4.4  5400
~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

SubDataset  "dataset 2":
                                           ____________________________________                                          
                                           *** TableDataset (Example table) ***                                          
                                                           META                                                          
+-------------+---------------+--------+--------+---------+--------------+--------+-------------------+
| name        | value         | unit   | type   | valid   | default      | code   | description       |
+=============+===============+========+========+=========+==============+========+===================+
| description | Example table |        | string | None    | UNKNOWN      | B      | Description of th |
|             |               |        |        |         |              |        | is dataset        |
+-------------+---------------+--------+--------+---------+--------------+--------+-------------------+
| shape       | (2, 5)        |        | tuple  | None    | ()           |        | Number of columns |
|             |               |        |        |         |              |        |  and rows.        |
+-------------+---------------+--------+--------+---------+--------------+--------+-------------------+
| type        | TableDataset  |        | string | None    | TableDataset | B      | Data Type identif |
|             |               |        |        |         |              |        | ication.          |
+-------------+---------------+--------+--------+---------+--------------+--------+-------------------+
| version     | 0.1           |        | string | None    | 0.1          | B      | Version of datase |
|             |               |        |        |         |              |        | t                 |
+-------------+---------------+--------+--------+---------+--------------+--------+-------------------+
| FORMATV     | 1.6.0.2       |        | string | None    | 1.6.0.2      | B      | Version of datase |
|             |               |        |        |         |              |        | t schema and revi |
|             |               |        |        |         |              |        | sion              |
+-------------+---------------+--------+--------+---------+--------------+--------+-------------------+
| listeners   | <No listener> |        |        |         |              |        |                   |
+-------------+---------------+--------+--------+---------+--------------+--------+-------------------+
                                                           DATA                                                          
                                                           ----                                                          
   Time    Energy
  (sec)      (eV)
-------  --------
      0       100
      1       102
      2       104
      3       106
      4       108

~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~


=========================================================================================================================
level 1, repr
                 *** CompositeDataset (test CD) ***                

-------------  --------------------------  ------------------------
a= 3.4         b= xy (2019-02-19T01:02:03  c= Invalid (IJK)
               .456789
               1929229360456789)
d= off (0b00)  m1= 2.3                     listeners= <No listener>
-------------  --------------------------  ------------------------


           Total 2 Sub-Datasets: "dataset 1", "dataset 2"          

SubDataset  "dataset 1":
                 *** ArrayDataset (arraydset 1) ***                

------------------------  ------------------------  ------------------
description= arraydset 1  shape= (3,)               type= ArrayDataset
unit= ev                  typecode= UNKNOWN         version= 0.1
FORMATV= 1.6.0.2          listeners= <No listener>
------------------------  ------------------------  ------------------

768  4.4  5400
~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

SubDataset  "dataset 2":
                *** TableDataset (Example table) ***               

--------------------------  ----------------  ------------------------
description= Example table  shape= (2, 5)     type= TableDataset
version= 0.1                FORMATV= 1.6.0.2  listeners= <No listener>
--------------------------  ----------------  ------------------------

   Time    Energy
  (sec)      (eV)
-------  --------
      0       100
      1       102
      2       104
      3       106
      4       108

~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~


===================================================================
level 2,
CompositeDataset(META: a, b, c, d, m1 DATA: dataset 1, dataset 2)"""
out_FineTime = """toString test
=========== format: "%Y-%m-%dT%H:%M:%S.%f" =======
FineTime
level=0 width=0: 2019-02-19T01:02:03.456789 TAI(1929229360456789) format=%Y-%m-%dT%H:%M:%S.%f
level=1 width=0: 2019-02-19T01:02:03.456789 (1929229360456789)
level=2 width=0: FineTime(2019-02-19T01:02:03.456789)
level=0 width=1: 2019-02-19T01:02:03.456789
1929229360456789
level=1 width=1: 2019-02-19T01:02:03.456789
1929229360456789
level=2 width=1: FineTime(2019-02-19
01:02:03.456789)
FineTime1
level=0 width=0: 2019-02-19T01:02:03.457000 TAI(67309323457) format=%Y-%m-%dT%H:%M:%S.%f
level=1 width=0: 2019-02-19T01:02:03.457000 (67309323457)
level=2 width=0: FineTime1(2019-02-19T01:02:03.457000)
level=0 width=1: 2019-02-19T01:02:03.457000
67309323457
level=1 width=1: 2019-02-19T01:02:03.457000
67309323457
level=2 width=1: FineTime1(2019-02-19
01:02:03.457000)
=========== format: "%Y" =======
FineTime
level=0 width=0: 2019 TAI(1929229360456789) format=%Y
level=1 width=0: 2019 (1929229360456789)
level=2 width=0: FineTime(2019)
level=0 width=1: 2019
1929229360456789
level=1 width=1: 2019
1929229360456789
level=2 width=1: FineTime(2019)
FineTime1
level=0 width=0: 2019 TAI(67309323457) format=%Y
level=1 width=0: 2019 (67309323457)
level=2 width=0: FineTime1(2019)
level=0 width=1: 2019
67309323457
level=1 width=1: 2019
67309323457
level=2 width=1: FineTime1(2019)
"""
